/*
 * Copyright (c) 2018 UPLEX Nils Goroll Systemoptimierung
 * All rights reserved
 *
 * Author: Geoffrey Simmons <geoffrey.simmons@uplex.de>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

package vcl

import (
	"bytes"
	"fmt"
	"io/ioutil"
	"os"
	"path/filepath"
	"testing"
)

func cmpGold(got []byte, goldfile string) (bool, error) {
	goldpath := filepath.Join("testdata", goldfile)
	gold, err := ioutil.ReadFile(goldpath)
	if err != nil {
		return false, err
	}
	return bytes.Equal(got, gold), nil
}

func TestMain(m *testing.M) {
	tmplDir := ""
	tmplEnv, exists := os.LookupEnv("TEMPLATE_DIR")
	if !exists {
		tmplDir = tmplEnv
	}
	if err := InitTemplates(tmplDir); err != nil {
		fmt.Printf("Cannot parse templates: %v\n", err)
		os.Exit(-1)
	}
	code := m.Run()
	os.Exit(code)
}

var dfbSpec = Spec{
	Version: "4.7.11",
	Services: map[string]Service{
		"bar-service": Service{
			Meta: Meta{
				Namespace:       "default",
				UID:             "47850e08-0791-11ea-b351-bab55b1b33a8",
				ResourceVersion: "514058",
			},
			Endpoints: map[string]Endpoint{
				"bar-64548cd7b6-j5b5w": Endpoint{
					Meta: Meta{
						Namespace:       "default",
						UID:             "d7d27ba8-0792-11ea-b351-bab55b1b33a8",
						ResourceVersion: "515214",
					},
					IP:   "172.22.225.13",
					Port: 8080,
				},
				"bar-64548cd7b6-fb2gb": Endpoint{
					Meta: Meta{
						Namespace:       "default",
						UID:             "d52a8e5b-0792-11ea-b351-bab55b1b33a8",
						ResourceVersion: "515183",
					},
					IP:   "172.22.225.49",
					Port: 8080,
				},
			},
		},
		"baz-service": Service{
			Meta: Meta{
				Namespace:       "default",
				UID:             "993323b2-077f-11ea-b351-bab55b1b33a8",
				ResourceVersion: "502259",
			},
			Endpoints: map[string]Endpoint{
				"baz-5ff49fcd4-hx7lj": Endpoint{
					Meta: Meta{
						Namespace:       "default",
						UID:             "efdb0528-0792-11ea-b351-bab55b1b33a8",
						ResourceVersion: "515894",
					},
					IP:   "172.22.225.8",
					Port: 8080,
				},
			},
		},
		"foo-service": Service{
			Meta: Meta{
				Namespace:       "default",
				UID:             "5145d363-079b-11ea-b351-bab55b1b33a8",
				ResourceVersion: "520824",
			},
			Endpoints: map[string]Endpoint{
				"foo-stateful-0": Endpoint{
					Meta: Meta{
						Namespace:       "default",
						UID:             "51712506-079b-11ea-b351-bab55b1b33a8",
						ResourceVersion: "520869",
					},
					IP:   "172.22.225.25",
					Port: 6081,
				},
				"foo-stateful-1": Endpoint{
					Meta: Meta{
						Namespace:       "default",
						UID:             "56f812d1-079b-11ea-b351-bab55b1b33a8",
						ResourceVersion: "520898",
					},
					IP:   "172.22.225.39",
					Port: 6081,
				},
			},
		},
	},
}

func TestTemplate(t *testing.T) {
	var buf bytes.Buffer
	gold := "backends.golden"
	if err := tmpl.Execute(&buf, dfbSpec); err != nil {
		t.Fatal("Execute():", err)
	}
	ok, err := cmpGold(buf.Bytes(), gold)
	if err != nil {
		t.Fatalf("Reading %s: %v", gold, err)
	}
	if !ok {
		t.Errorf("Generated VCL does not match gold file: %s", gold)
		if testing.Verbose() {
			t.Logf("Generated: %s", buf.String())
		}
	}
}
